
import 'dart:io';

import 'package:chobi_admob_flutter/chobi_admob.dart';
import 'package:chobi_admob_flutter/chobi_admob_flutter_platform_interface.dart';
import 'package:flutter/cupertino.dart';
import 'package:google_mobile_ads/google_mobile_ads.dart';
import 'package:hooks_riverpod/hooks_riverpod.dart';


// TODO: create the class holds banner ad status -> available, loaded, ad object, and so on.

// TODO: admob initialize after UMP CHECK

/*
BannerState _copyBannerState(
  BannerState src,
    {
      bool? isBannerAvailable,
      bool? isBannerAdLoaded,
      BannerAd? Function()? bannerAdGetter,
      bool? appTrackingConfirmed,
    }) {

  final ad = (bannerAdGetter != null) ? bannerAdGetter() : src.bannerAd;
  return BannerState(
    isBannerAvailable: isBannerAvailable ?? src.isBannerAvailable,
    isBannerAdLoaded: isBannerAdLoaded ?? src.isBannerAdLoaded,
    bannerAd: ad,
    appTrackingConfirmed: appTrackingConfirmed ?? src.appTrackingConfirmed,
  );
}

 */

class BannerState {

  final bool isBannerAvailable;
  final bool isBannerAdLoaded;

  final BannerAd? bannerAd;
  //final Ad? ad;

  final bool appTrackingConfirmed;

  BannerState({
    this.isBannerAvailable = false,
    this.isBannerAdLoaded = false,
    this.bannerAd,
    //this.ad,
    this.appTrackingConfirmed = false,
  });
}

class ChobiAdMobBannerAd extends ConsumerStatefulWidget {

  final String? androidBannerId;
  final String? iosBannerId;
  final String lastClickedPrefsKey;
  final List<String>? testDeviceIds;
  final bool isDebug;
  final int adIntervalTimeMs;

  final Function(ConsentStatus status)? additionalConsentCheckFinished;
  final Function(FormError error)? additionalConsentError;

  final BannerAdListener? bannerAdListener;

  //late final StateProvider<bool> isAdAvailableProvider;
  //late final StateProvider<bool> isAdLoadedProvider;
  //late final StateProvider<BannerAd?> bannerAdProvider;

  /*
  final StateProvider<bool>? isAdAvailableProvider;
  final StateProvider<bool>? isAdLoadedProvider;
  final StateProvider<BannerAd?>? bannerAdProvider;

   */

  //final StateProvider<TrackingStatus>? appTrackingStatusProvider;

  final StateProvider<BannerState>? bannerStateProvider;

  final bool needToConfirmIosAppTracking;

  final Widget Function(BuildContext context, BannerAd? ad) builder;

  const ChobiAdMobBannerAd({
    super.key,
    this.androidBannerId,
    this.iosBannerId,
    required this.lastClickedPrefsKey,
    this.testDeviceIds,
    this.isDebug = true,
    this.adIntervalTimeMs = 5 * 60 * 1000,
    this.additionalConsentCheckFinished,
    this.additionalConsentError,
    this.bannerAdListener,

    //this.isAdAvailableProvider,
    //this.isAdLoadedProvider,
    //this.bannerAdProvider,
    this.bannerStateProvider,

    this.needToConfirmIosAppTracking = true,
    //this.appTrackingStatusProvider,
    required this.builder,
  });

  //{
  //  this.isAdAvailableProvider = isAdAvailableProvider ?? StateProvider((ref) => false);
  //  this.isAdLoadedProvider = isAdLoadedProvider ?? StateProvider((ref) => false);
  //  this.bannerAdProvider = bannerAdProvider ?? StateProvider((ref) => null);
  //}

  @override
  ConsumerState<ChobiAdMobBannerAd> createState() => ChobiAdMobBannerAdState();
}


const _blankBox = SizedBox.shrink(key: ValueKey('chobi_admob_banner_ad_blank_box'),);

class ChobiAdMobBannerAdState extends ConsumerState<ChobiAdMobBannerAd> with AutomaticKeepAliveClientMixin {

  late ChobiAdMob? _chobiAdMob;

  //final _iosAppTrackingConfirmedProvider = StateProvider((ref) => false);

  //late final StateProvider<bool> _isAdAvailableProvider;
  //late final StateProvider<bool> _isAdLoadedProvider;
  //late final StateProvider<BannerAd?> _bannerAdProvider;
  late final StateProvider<BannerState> _bannerStateProvider;

  void _updateBannerState(WidgetRef ref, {
    bool? isBannerAvailable,
    bool? isBannerAdLoaded,
    BannerAd? Function()? bannerAdGetter,
    bool? appTrackingConfirmed,
  }) {
    final oldState = ref.read(_bannerStateProvider);
    ref.read(_bannerStateProvider.notifier).state = BannerState(
      isBannerAvailable: isBannerAvailable ?? oldState.isBannerAvailable,
      isBannerAdLoaded: isBannerAdLoaded ?? oldState.isBannerAdLoaded,
      bannerAd: (bannerAdGetter != null) ? bannerAdGetter() : oldState.bannerAd,
      appTrackingConfirmed: appTrackingConfirmed ?? oldState.appTrackingConfirmed,
    );
  }

  @override
  bool get wantKeepAlive => true;

  @override
  void initState() {
    super.initState();

    _bannerStateProvider = widget.bannerStateProvider ?? StateProvider((ref) => BannerState());

    //_isAdAvailableProvider = widget.isAdAvailableProvider ?? StateProvider((ref) => false);
    //_isAdLoadedProvider = widget.isAdLoadedProvider ?? StateProvider((ref) => false);
    //_bannerAdProvider = widget.bannerAdProvider ?? StateProvider((ref) => null);

    WidgetsBinding.instance.addPostFrameCallback((_) async {
      if (Platform.isIOS && widget.needToConfirmIosAppTracking) {
        await ChobiAdmobFlutterPlatform.instance.confirmAppTrackingTransparency();
      }

      //ref.read(_iosAppTrackingConfirmedProvider.notifier).state = true;
      _updateBannerState(ref, appTrackingConfirmed: true);

      await ChobiAdMob.loadLastClickedTimeFromPrefs(prefKey: widget.lastClickedPrefsKey).then((value) {
        _chobiAdMob = ChobiAdMob(
          adLastClickedTime: value,
          lastClickedPrefsKey: widget.lastClickedPrefsKey,
          isDebug: widget.isDebug,
          testDeviceIds: widget.testDeviceIds,
          androidId: widget.androidBannerId,
          iosId: widget.iosBannerId,
          androidTestId: ChobiAdMob.androidTestAdaptiveBannerId,
          iosTestId: ChobiAdMob.iosTestAdaptiveBannerId,
          adIntervalTimeMs: widget.adIntervalTimeMs,
        )..onConsentCheckFinished =  (status) {
          _onConsentCheckFinished(status);
          widget.additionalConsentCheckFinished?.call(status);
        }..onConsentCheckError = (err) {
          _onConsentError(err);
          widget.additionalConsentError?.call(err);
        }..checkPermissionAndStartAdProcess();
      });
    });
  }


  @override
  void dispose() {
    Future.sync(() async {
      await _chobiAdMob?.stopAdProcess();
      await disposeBanner();
    });

    super.dispose();
  }

  @protected
  Future<void> loadBanner() async {
    //if (!ref.read(widget.isAdAvailableProvider)) {
    //if (!ref.read(_isAdAvailableProvider)) {
    if (!ref.read(_bannerStateProvider).isBannerAvailable) {
      debugPrint('AD not available');
      return;
    }

    final bannerId = _chobiAdMob?.getAdId();
    debugPrint('banner id = $bannerId');

    if (bannerId == null) {
      debugPrint('banner id is null');
      return;
    }

    final cWidth = MediaQuery.of(context).size.width.truncate();
    final adSize = await AdSize.getCurrentOrientationAnchoredAdaptiveBannerAdSize(cWidth);

    if (adSize == null) {
      debugPrint('adSize is null');
      return;
    }

    final ad = BannerAd(
      size: adSize,
      adUnitId: bannerId,
      listener: generateBannerListener(),
      request: const AdRequest(),
    );

    //ref.read(widget.bannerAdProvider.notifier).state = ad;
    //ref.read(_bannerAdProvider.notifier).state = ad;
    _updateBannerState(ref, bannerAdGetter: () => ad);

    await ad.load();
  }

  @protected
  Future<void> disposeBanner() async {
    /*
    await ref.read(widget.bannerAdProvider)?.dispose();
    ref.read(widget.bannerAdProvider.notifier).state = null;
    ref.read(widget.isAdAvailableProvider.notifier).state = false;
    ref.read(widget.isAdLoadedProvider.notifier).state = false;

     */

    ref.read(_bannerStateProvider).bannerAd?.dispose();
    ref.read(_bannerStateProvider.notifier).state = BannerState();

    /*
    await ref.read(_bannerAdProvider)?.dispose();
    ref.read(_bannerAdProvider.notifier).state = null;
    ref.read(_isAdAvailableProvider.notifier).state = false;
    ref.read(_isAdLoadedProvider.notifier).state = false;

     */
  }

  @protected
  BannerAdListener generateBannerListener() {
    return BannerAdListener(
        onAdClosed: widget.bannerAdListener?.onAdClosed,
        onAdImpression: widget.bannerAdListener?.onAdImpression,
        onAdOpened: widget.bannerAdListener?.onAdOpened,
        onAdWillDismissScreen: widget.bannerAdListener?.onAdWillDismissScreen,
        onPaidEvent: widget.bannerAdListener?.onPaidEvent,

        onAdLoaded: (ad) {
          //ref.read(widget.bannerAdProvider.notifier).state = ad as BannerAd?;
          //ref.read(widget.isAdLoadedProvider.notifier).state = true;

          //ref.read(_isAdLoadedProvider.notifier).state = true;
          _updateBannerState(ref, isBannerAdLoaded: true);

          widget.bannerAdListener?.onAdLoaded?.call(ad);
        },
        onAdFailedToLoad: (ad, err) {
          debugPrint('admob banner laod error: ${err.message}');
          //ref.read(widget.isAdLoadedProvider.notifier).state = false;

          //ref.read(_isAdLoadedProvider.notifier).state = false;
          _updateBannerState(ref, isBannerAdLoaded: false);

          widget.bannerAdListener?.onAdFailedToLoad?.call(ad, err);
          //ref.read(widget.bannerAdProvider.notifier).state = null;
          disposeBanner();
        },
        onAdClicked: (ad) async {
          await _chobiAdMob?.updateLastClickedTime();
          await disposeBanner();
          await _chobiAdMob?.checkPermissionAndStartAdProcess();
          widget.bannerAdListener?.onAdClicked?.call(ad);
        }
    );
  }


  void _onConsentCheckFinished(ConsentStatus status) {
    if (status != ConsentStatus.required) {
      //ref.read(widget.isAdAvailableProvider.notifier).state = true;
      //ref.read(_isAdAvailableProvider.notifier).state = true;
      _updateBannerState(ref, isBannerAvailable: true);
      loadBanner();
    }
  }

  void _onConsentError(FormError error) {
    //ref.read(widget.isAdAvailableProvider.notifier).state = false;
    //ref.read(_isAdAvailableProvider.notifier).state = false;
    _updateBannerState(ref, isBannerAvailable: false);

    debugPrint('UMP FORM ERROR[${error.errorCode}]: ${error.message}');
  }


  @override
  Widget build(BuildContext context) {
    super.build(context);

    return Consumer(
      builder: (atCtx, atRef, atChild) {
        //final isAppTrackingChecked = atRef.watch(_iosAppTrackingConfirmedProvider);
        final isAppTrackingChecked = atRef.watch(_bannerStateProvider.select((value) => value.appTrackingConfirmed));
        
        return isAppTrackingChecked ? Consumer(
          builder: (adCtx, adRef, adChild) {
            //final ad = adRef.watch(widget.bannerAdProvider);
            //final ad = adRef.watch(_bannerAdProvider);
            final ad = adRef.watch(_bannerStateProvider.select((value) => value.bannerAd));

            return widget.builder(adCtx, ad);
          },
        ) : _blankBox;
      },
    );

    /*
    final isAppTrackingChecked = ref.watch(_iosAppTrackingConfirmedProvider);
    final ad = ref.watch(widget.bannerAdProvider);
    return isAppTrackingChecked
        ? widget.builder(context, ad)
        : const SizedBox.shrink(key: ValueKey('chobi_admob_banner_ad_blank_box'),);

     */
  }
}

